#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <math.h>

#  define OC1 PB1
#  define DDROC DDRB
#  define OCR OCR1A
#  define PWM10 WGM10
#  define PWM11 WGM11

volatile uint16_t xtimer;
volatile uint16_t timer0;
volatile uint8_t status;

SIGNAL(SIG_OVERFLOW0)
{
  timer0++;
  TCNT0=96;
}

//rising edge PWM signal
SIGNAL(SIG_INTERRUPT0) {
  //record the length of the positive pwm signal in xtimer
  if(timer0<270) xtimer=timer0;
  // if timer0 is greater than approximately 263 (at 20 us per interrupt) than the pulse was missed
  status=0;
}

// falling edge PWM signal
SIGNAL(SIG_INTERRUPT1) {
  // Zero timer0 to count the length of the positive pulse
  // The PWM signal from the clock is reversed due to the optoisolator
  // So the rising edge of the PWM signal is actually the faling edge at the microcontroller
  timer0=0;
  status=1;
}

void
ioinit (void)
{
    // Timer1 does ~16-bit PWM
    TCCR1A = _BV (COM1B1) | _BV (WGM11) | _BV (COM1A1); // | _BV(COM1A0);
    TCCR1B = _BV (WGM13) | _BV (WGM12) | _BV (CS10);

    ICR1 = 65535;

    //Timer0 counts
    TCCR0 = _BV (CS00);
    TCNT0 = 96;
    timer0=0;

    // set PWM value to 0
    OCR = 0;

    /* enable OC1 and PB2 as output */
    DDROC = _BV (OC1);
    //DDRC = _BV (PC0);

    //enable external interrupts
    GICR = _BV (INT1) | _BV (INT0);
    //INT0 is rising edge, INT1 is falling edge
    MCUCR = _BV (ISC01) | _BV (ISC00) | _BV(ISC11);

    xtimer=0;
    status=0;

    //enable timers
    timer_enable_int (_BV (TOIE1) | _BV (TOIE0) );

    /* enable interrupts */
    sei ();
}

int
main (void)
{
    #define B -0.00508
    #define C -11.09
    #define D 0.474818

    int i,on=0;
    unsigned int x[100], y;
    long t[100], u;
    ioinit ();

    for (;;) {

      //average xtimer over samples because it jumps around alot
      for(i=99;i>0;i--) {
        x[i] = x[i-1];
      }
      x[0] = xtimer;
      y=0;
      for(i=0;i<100;i++) {
        y = y+x[i];
      }
      y=y/100;
      //divide by 5 because the clock has 120 us resolution on its PWM
      y=y/5;

      for(i=99;i>0;i--) {
        t[i] = t[i-1];
      }

      if(timer0>270 && status == 1) {
        t[0] = 0;
        on = 1;
      }
      else if(timer0>270 && status == 0) {
        t[0] = 65535;
        xtimer=0;
        on=0;
      }
      else if(on==0){
        t[0] = 65535*(1-exp((B*y + D)*y + C));
        if(t[0]>65535) t[0] = 65535;
        if(t[0]<0) t[0] = 0;
      }


      u=0;
      for(i=0;i<100;i++) {
        u=u+t[i];
      }
      OCR = u/100;


    }

    return (0);
}
