/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <joerg@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.        Joerg Wunsch
 * ----------------------------------------------------------------------------
 *
 * Simple AVR demonstration.  Controls a LED that can be directly
 * connected from OC1/OC1A to GND.  The brightness of the LED is
 * controlled with the PWM.  After each period of the PWM, the PWM
 * value is either incremented or decremented, that's all.
 *
 * $Id: demo.c,v 1.4 2004/07/21 21:03:07 joerg_wunsch Exp $
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#  define OC1 PB1
#  define DDROC DDRB
#  define OCR OCR1A
#  define PWM10 WGM10
#  define PWM11 WGM11

#if defined(COM11)
#  define XCOM11 COM11
#elif defined(COM1A1)
#  define XCOM11 COM1A1
#else
#  error "need either COM1A1 or COM11"
#endif

#define setbit(x,y)   (x |= (1 << (y)))
#define clearbit(x,y) (x &= ~(1 << (y)))
#define checkbit(x,y) (x & (1 << (y)))


#define RED_LED_ON setbit(PORTB, 7)
#define RED_LED_OFF clearbit(PORTB, 7)
#define GREEN_LED_ON setbit(PORTB, 6)
#define GREEN_LED_OFF clearbit(PORTB, 6)

enum { UP, DOWN };

volatile uint16_t timer1; /* Note [1] */
volatile uint8_t direction;
volatile uint16_t pwm;
volatile uint16_t timer0;

/*
SIGNAL (SIG_OVERFLOW1)
{
    switch (direction)
    {
        case UP:
            if (++pwm == 1023)
                direction = DOWN;
            break;

        case DOWN:
            if (--pwm == 0)
                direction = UP;
            break;
    }

    OCR = pwm;
    if(++timer1==2) {
      timer1=0;
    }

} */

SIGNAL (SIG_OVERFLOW0)
{
  if(++timer0==2) {
    timer0=0;
  }
  TCNT0=96;
}


// turn yellow leds on/off
void led(char id, char state) {
  if(id < 2) {
    if(state) setbit(PORTD, id);
    else      clearbit(PORTD, id);
  }
  else if(id < 7) {
    if(state) setbit(PORTD, id+1);
    else      clearbit(PORTD, id+1);
  }
  else if(id < 10) {
    if(state) setbit(PORTB, id-7);
    else      clearbit(PORTB, id-7);
  }
}


void
ioinit (void) /* Note [5] */
{
    // Timer0
    TCCR0 = _BV (CS00);
    TCNT0=100;

    //timer_enable_int (_BV (TOIE0));

    /* tmr1 is 10-bit PWM */
    TCCR1A = 5; _BV (WGM10) | _BV (WGM11) | _BV (XCOM11);

    /* tmr1 running on full MCU clock */
    TCCR1B = 1; // _BV (CS10);

    /* set PWM value to 0 */
    //OCR = 0;

    /* enable OC1 and PB2 as output */
    //DDROC = _BV (OC1);

    timer_enable_int (_BV (TOIE0) );
    //timer_enable_int (_BV (TOIE1) | _BV (TOIE0) );

    // port init
    PORTB = 0x38;
    DDRB  = 0xC7;
    PORTC = 0x7F;
    DDRC  = 0x00;
    PORTD = 0x04;
    DDRD  = 0xFB;
    //DDRD = 0x00;

    /* enable interrupts */
    sei ();
}

int main (void)
{
    unsigned short t;

    ioinit ();

    /* loop forever, the interrupts are doing the rest */

    for (;;){  /* Note [6] */
      if(timer0==1) {
        led(0,1);
        //RED_LED_ON;
      }
      if(timer0==0) {
        led(0,0);
      }


    }


    return (0);
}
